package com.starwsn.test;

import org.eclipse.paho.client.mqttv3.*;
import org.eclipse.paho.client.mqttv3.persist.MemoryPersistence;

import java.util.UUID;

/**
 * mqtt服务
 * @author 郝晓全
 */
public class MqttServices {
//	private static String host = "tcp://iot.starwsn.com:1883"; // 正式服务地址
	private static String host = "tcp://119.23.104.41:1883"; // 测试服务地址
	private static String userName = "admin"; // 用户名
	private static String passWord = "password"; // 密码
	private static int keepAlive = 10; // 超时时间
	private static int keepAliveInterval = 20; // 会话心跳时间
	private static String clientId = "starwsn-mqtt-test-" + UUID.randomUUID(); // 客户端ID,需修改,推荐格式  公司英文（或拼音）-项目工程名-随机数
	private static MqttClient client; // 客户端对象

	/**
	 * 连接服务器
	 * @param cleanSession 是否清除session
	 */
	public static boolean connect(boolean cleanSession) {
		try {
			client = new MqttClient(host, clientId, new MemoryPersistence());
			MqttConnectOptions options = new MqttConnectOptions();
			options.setCleanSession(cleanSession);
			options.setUserName(userName);
			options.setPassword(passWord.toCharArray());
			// 设置超时时间
			options.setConnectionTimeout(keepAlive);
			// 设置会话心跳时间
			options.setKeepAliveInterval(keepAliveInterval);
			client.setCallback(new MessageCallback());
			client.connect(options);
			System.out.println("mqtt服务连接成功！！！");
			return true;
		} catch (Exception e) {
			System.out.println("mqtt服务连接失败！！！");
			return false;
		}
	}
	
	/**
	 * 断开客户端连接
	 */
	public static boolean disConnect() {
		try {
			client.disconnect();
			client.close();
			System.out.println("mqtt服务断开成功！！！");
			return true;
		} catch (MqttException e) {
			System.out.println("mqtt服务断开成功！！！");
			return false;
		}
	}

	/**
	 * 主题订阅
	 * @param topics 主题名称数组
	 * @param Qos 服务质量数组（与主题名称一一匹配）
	 * @return
	 */
	public static boolean subScription(String[] topics,int[] Qos) {
		try {
			client.subscribe(topics, Qos);
			String loginfo = "mqtt主题{";
			for(int i=0;i<topics.length;i++){
				loginfo += topics[i];
				if(i != topics.length-1)
					loginfo += ",";
			}
			loginfo += "}订阅成功！！！";
			System.out.println(loginfo);
			return true;
		} catch (Exception e) {
			String loginfo = "mqtt主题{";
			for(int i=0;i<topics.length;i++){
				loginfo += topics[i];
				if(i != topics.length-2)
					loginfo += ",";
			}
			loginfo += "}订阅失败！！！";
			System.out.println(loginfo);
			return false;
		}
	}
	
	/**
	 * 主题订阅
	 * @param topic 主题名称
	 * @param Qos  服务质量
	 * @return
	 */
	public static boolean subScription(String topic,int Qos) {
		try {
			int[] _Qos = { Qos };
			String[] _topic = { topic };
			client.subscribe(_topic, _Qos);
			System.out.println("mqtt主题{"+topic+"}订阅成功！！！");
			return true;
		} catch (Exception e) {
			System.out.println("mqtt主题{"+topic+"}订阅失败！！！");
			return false;
		}
	}
	
	/**
	 * 主题订阅，默认服务质量为0
	 * @param topic 主题名称
	 * @return
	 */
	public static boolean subScription(String topic) {
		try {
			int[] _Qos = { 0 };
			String[] _topic = { topic };
			client.subscribe(_topic, _Qos);
			System.out.println("mqtt主题{"+topic+"}订阅成功！！！");
			return true;
		} catch (Exception e) {
			System.out.println("mqtt主题{"+topic+"}订阅失败！！！");
			return false;
		}
	}

	/**
	 * 取消主题订阅
	 * @param topics 主题名称
	 * @return
	 */
	public static boolean unSubScription(String[] topics) {
		String topic = "";
		try {
			for(int i=0;i<topics.length;i++){
				topic = topics[i];
				client.unsubscribe(topic);
				System.out.println("mqtt主题{" + topic + "}取消订阅成功！！！");
			}
			return true;
		} catch (Exception e) {
			System.out.println("mqtt主题{" + topic + "}取消订阅失败！！！");
			return false;
		}
	}
	
	/**
	 * 取消主题订阅
	 * @param topic 主题名称
	 * @return
	 */
	public static boolean unSubScription(String topic) {
		try {
			client.unsubscribe(topic);
			System.out.println("mqtt主题{" + topic + "}取消订阅成功！！！");
			return true;
		} catch (Exception e) {
			System.out.println("mqtt主题{" + topic + "}取消订阅失败！！！");
			return false;
		}
	}
	
	/**
	 * 推送消息
	 * @param topicName  主题名称
	 * @param payload  消息内容
	 * @return
	 */
	public static boolean publish(String topicName,byte[] payload){
		try {
			MqttTopic topic = client.getTopic(topicName);
			MqttMessage message = new MqttMessage();
			message.setQos(1);
			message.setRetained(false);
			message.setPayload(payload);
			MqttDeliveryToken token = topic.publish(message);
			System.out.println("mqtt消息发送结果等待。。。。。");
			token.waitForCompletion();
			System.out.println("mqtt消息发送结果："+token.isComplete());
			return token.isComplete();
		} catch (Exception e) {
			e.printStackTrace();
			return false;
		}
	}
}
